@@ -6,18 +6,19 @@
 
 package MCE::Shared::Hash;
 
-use 5.010001;
 use strict;
 use warnings;
 
+use 5.010001;
+
 no warnings qw( threads recursion uninitialized numeric );
 
-our $VERSION = '1.817';
+our $VERSION = '1.818';
 
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
 
 use MCE::Shared::Base;
-use parent -norequire, 'MCE::Shared::Base::Common';
+use base 'MCE::Shared::Base::Common';
 use bytes;
 
 use overload (
@@ -311,7 +312,7 @@ MCE::Shared::Hash - Hash helper class
 
 =head1 VERSION
 
-This document describes MCE::Shared::Hash version 1.817
+This document describes MCE::Shared::Hash version 1.818
 
 =head1 DESCRIPTION
 
